<?php

namespace Ems\Services;

use Doctrine\ORM\EntityManager;

use Util\Repositories;
use Util\Environment;
use Util\Messages;
use Ems\Util\ArrayUtil;

use Ems\Services\ParceiroNegocioService;
use Ems\Repository\OperadorRepository;
use Ems\Repository\ParamOrgRepository;
use Geral\Util\NovoCodigo;

class OperadorService {

    /** @var OperadorRepository */
    protected $operadorRepository;

    /** @var Environment */
    protected $environment;

    /** @var ParamOrgRepository */
    protected $paramOrgRepository;

    public function __construct(
        EntityManager $entityManager,
        Environment $environment,
        ParceiroNegocioService $parceiroNegocioService,
        OperadorRepository $operadorRepository,
        ParamOrgRepository $paramOrgRepository
    ) {
        $this->entityManager            = $entityManager;
        $this->environment              = $environment;
        $this->parceiroNegocioService   = $parceiroNegocioService;
        $this->operadorRepository       = $operadorRepository;
        $this->paramOrgRepository       = $paramOrgRepository;
    }


    /**
     *  Função que retorna dados úteis do operador.
     *
     *  @param string $cdoperador
     *
     *  @return Array $dadosOperador
     *      'NMOPERADOR'    => $nmoperadornotificacao,
     *      'EMAILOPEADOR'  => $emailoperadornotificacao
     *
     *  @throws Exception
     */
    public function getDadosOperador($cdoperador, $nrorg = null) {
        if (empty($nrorg)) {
            $nrorg = $this->environment->getNrorg();
        }

        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('nrorg' => $nrorg, 'cdoperador' => $cdoperador));

        if (!empty($operador)) {
            $parceiroNegocioOperador = $this->entityManager->getRepository(Repositories::PARCNEGOCIO)->findOneBy(array('nrorg' => $nrorg, 'nrparcnegocio' => $operador->getNrparcnegocio()));

            if (!empty($parceiroNegocioOperador)) {
                $nmoperadornotificacao = $parceiroNegocioOperador->getNmprincipalparc();
                $emailoperadornotificacao = $this->parceiroNegocioService->getEmailParcNegocio($operador->getNrparcnegocio());
            } else {
                throw new \Exception(Messages::PARCNEGOCIO_OPERADOR_NOT_FOUND);
            }
        } else {
            throw new \Exception(Messages::OPERADOR_NOT_FOUND);
        }

        $dadosOperador = array(
            'NMOPERADOR' => $nmoperadornotificacao,
            'EMAILOPERADOR' => $emailoperadornotificacao
        );

        return $dadosOperador;
    }

    public function getPerfisDeAcesso(
        string $cdOperador
    ): array {
        $nrOrg = (int) $this->environment->getNrorg();

        $params = [
            'P_NRORG'      => $nrOrg,
            'P_CDOPERADOR' => $cdOperador
        ];

        $perfis = $this->operadorRepository->getPerfisDeAcesso($params);

        $nrPerfis = array_map(function($perfil) {
            return $perfil['NRPERFILACESSO'];
        }, $perfis);

        return $nrPerfis;
    }

    public function operadorIsDP(
        string $cdOperador
    ): bool {
        $nrOrg = (int) $this->environment->getNrorg();

        $nrPerfis = $this->getPerfisDeAcesso($cdOperador);

        $paramOrg = $this->paramOrgRepository->getParamOrg($nrOrg);

        $stringPerfisDP = $paramOrg['NRPERFACESSADM'];
        $stringPerfisDP = is_null($stringPerfisDP) ? '' : $stringPerfisDP;
        
        $perfisDP = ArrayUtil::fromStringSQL($stringPerfisDP, ArrayUtil::COMMA_ARRAY_STRING_SEPARATOR, false);
        
        $intersection = ArrayUtil::intersection($nrPerfis, $perfisDP);

        $isDP = count($intersection) > 0;

        return $isDP;
    }

    public function getOperadoresDp() {
        $nrorg = $this->environment->getNrorg();
        return $this->operadorRepository->getOperadoresDp($nrorg);
    }

    public function get(
        string $cdOperador
    ) {
        $nrOrg = (int) $this->environment->getNrOrg();

        $operador = $this->operadorRepository->get(
            $nrOrg, $cdOperador
        );

        return $operador;
    }

    public static function generatePassword(
        string $nmPessoa
    ): string {
        return str_repeat(strtolower(substr($nmPessoa,0,1)),5);
    }

    public static function encryptPassword(
        string $password
    ): string {
        return md5($password);
    }

    public function insert(
        array $row,
        $commitPK = true
    ): string {
        $nrOrg = (int) $this->environment->getNrOrg();
        $cdOperInclusao = (string) $this->environment->getCdOperador();

        $params = [];

        if (empty($row['CDOPERADOR'])) {
            $params['CDOPERADOR'] = NovoCodigo::geraCodigo($nrOrg, 'OPERADOR', 1, 10, false, $commitPK);
        }

        $params['CDPAIS']               = $row['CDPAIS'] ?? null;
        $params['SGESTADO']             = $row['SGESTADO'] ?? null;
        $params['CDMUNICIPIO']          = $row['CDMUNICIPIO'] ?? null;
        $params['NMOPERADOR']           = $row['NMOPERADOR'] ?? null;
        $params['NMACESSO']             = $row['NMACESSO'] ?? null;
        $params['NMCOMPLOPER']          = $row['NMCOMPLOPER'] ?? null;
        $params['DTNASCIMENTO']         = $row['DTNASCIMENTO'] ?? null;
        $params['DSENDEOPER']           = $row['DSENDEOPER'] ?? null;
        $params['NMBAIROPER']           = $row['NMBAIROPER'] ?? null;
        $params['NRCEPOPER']            = $row['NRCEPOPER'] ?? null;
        $params['NRTELEOPER']           = $row['NRTELEOPER'] ?? null;
        $params['CDSENHOPER']           = $row['CDSENHOPER'] ?? null;
        $params['CDCLIENTE']            = $row['CDCLIENTE'] ?? null;
        $params['NROPERREDE']           = $row['NROPERREDE'] ?? null;
        $params['CDEMPRESA']            = $row['CDEMPRESA'] ?? null;
        $params['CDFUNC']               = $row['CDFUNC'] ?? null;
        $params['NRRAMALOPER']          = $row['NRRAMALOPER'] ?? null;
        $params['IDLIGEXTOPER']         = $row['IDLIGEXTOPER'] ?? 'N';
        $params['IDSENHVARDIA']         = $row['IDSENHVARDIA'] ?? null;
        $params['DSEMAILOPER']          = $row['DSEMAILOPER'] ?? null;
        $params['IDOPERMASTER']         = $row['IDOPERMASTER'] ?? null;
        $params['IDOPERACESSO']         = $row['IDOPERACESSO'] ?? null;
        $params['CDSENEMAILOP']         = $row['CDSENEMAILOP'] ?? null;
        $params['IDOPERATIVO']          = $row['IDOPERATIVO'] ?? null;
        $params['CDSETOR']              = $row['CDSETOR'] ?? null;
        $params['IDFUNINIOPER']         = $row['IDFUNINIOPER'] ?? null;
        $params['HRINIAGEOPER']         = $row['HRINIAGEOPER'] ?? null;
        $params['HRFIMAGEOPER']         = $row['HRFIMAGEOPER'] ?? null;
        $params['CDCARGO']              = $row['CDCARGO'] ?? null;
        $params['DTULTATUSENH']         = $row['DTULTATUSENH'] ?? null;
        $params['IDTPOPERADOR']         = $row['IDTPOPERADOR'] ?? null;
        $params['IDSTATUSENHA']         = $row['IDSTATUSENHA'] ?? null;
        $params['DTULTACESOPER']        = $row['DTULTACESOPER'] ?? null;
        $params['NRCELULAROPER']        = $row['NRCELULAROPER'] ?? null;
        $params['IDATUOPERADOR']        = $row['IDATUOPERADOR'] ?? 'S';
        $params['CDBANCOPER']           = $row['CDBANCOPER'] ?? null;
        $params['CDAGENOPER']           = $row['CDAGENOPER'] ?? null;
        $params['CDCCOROPER']           = $row['CDCCOROPER'] ?? null;
        $params['CDOPERESPLIBSEN']      = $row['CDOPERESPLIBSEN'] ?? null;
        $params['NRORG']                = $row['NRORG'] ?? $nrOrg;
        $params['NRORGINCLUSAO']        = $row['NRORGINCLUSAO'] ?? $nrOrg;
        $params['CDOPERINCLUSAO']       = $row['CDOPERINCLUSAO'] ?? $cdOperInclusao;
        $params['NRORGULTATU']          = $row['NRORGULTATU'] ?? $nrOrg;
        $params['CDOPERULTATU']         = $row['CDOPERULTATU'] ?? $row['CDOPERINCLUSAO'] ?? $cdOperInclusao;
        $params['NRPARCNEGOCIO']        = $row['NRPARCNEGOCIO'] ?? null;
        $params['CDSENHAOPERWEB']       = $row['CDSENHAOPERWEB'] ?? null;
        $params['NRPERFILSELECIONADO']  = $row['NRPERFILSELECIONADO'] ?? null;
        $params['NRORGSELECIONADA']     = $row['NRORGSELECIONADA'] ?? null;
        $params['CDFILIALSELECIONADA']  = $row['CDFILIALSELECIONADA'] ?? null;
        $params['CDPROCHABLANOPE']      = $row['CDPROCHABLANOPE'] ?? null;
        $params['CDPROCHABAPROPE']      = $row['CDPROCHABAPROPE'] ?? null;
        $params['CDPROCHABFECOPE']      = $row['CDPROCHABFECOPE'] ?? null;
        $params['IDCADPRAPROVORE']      = $row['IDCADPRAPROVORE'] ?? 'N';
        $params['IDCADPRPROCEORE']      = $row['IDCADPRPROCEORE'] ?? 'N';
        $params['IDATIVO']              = $row['IDATIVO'] ?? 'S';
        $params['NRORGTRAB']            = $row['NRORGTRAB'] ?? $nrOrg;
        $params['NRESTRUTTRAB']         = $row['NRESTRUTTRAB'] ?? null;
        $params['NRORGFILIAL']          = $row['NRORGFILIAL'] ?? null;
        $params['IDIDIOMA']             = $row['IDIDIOMA'] ?? null;
        $params['SGOPERADOR']           = $row['SGOPERADOR'] ?? null;
        $params['CDSENHAOPERMD5']       = $row['CDSENHAOPERMD5'] ?? null;
        $params['DSHASHALTERASENHA']    = $row['DSHASHALTERASENHA'] ?? null;
        $params['DSEMAILOPER_BKP']      = $row['DSEMAILOPER'] ?? null;
        $params['DSCRIPTORECSENHA']     = $row['DSCRIPTORECSENHA'] ?? null;
        $params['IDCHEFERECEITAM']      = $row['IDCHEFERECEITAM'] ?? 'N';
        $params['USR_NRORDEMSETOR']     = $row['USR'] ?? null;
        $params['NROPERREDE_BKP']       = $row['NROPERREDE'] ?? null;
        $params['IDOPERSAAS']           = $row['IDOPERSAAS'] ?? 'N';
        $params['DSIMAGEMOPER']         = $row['DSIMAGEMOPER'] ?? null;
        $params['NRORGOPER']            = $row['NRORGOPER'] ?? $nrOrg;
        $params['IDOPERMASTERZMART']    = $row['IDOPERMASTERZMART'] ?? 'N';
        $params['IDOPERMASTERCP']       = $row['IDOPERMASTERCP'] ?? 'N';
        $params['CDSENHAOPERSUP']       = $row['CDSENHAOPERSUP'] ?? null;
        $params['NRORGSUP']             = $row['NRORGSUP'] ?? null;
        $params['DTLIMACESSOSUP']       = $row['DTLIMACESSOSUP'] ?? null;

        $this->operadorRepository->insert($params);

        return $params['CDOPERADOR'];
    }
}